﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

namespace chart
{
    public partial class MainForm : Form
    {
        // 時間計測用ストップウォッチ
        Stopwatch stopWatch = null;

        // 相場データ
        List<Tuple<string, double>> data = new List<Tuple<string,double>>();

        // グラフ描画用のペン
        Pen pen = new Pen(new SolidBrush(Color.Black));
        Pen redPen = new Pen(new SolidBrush(Color.Red));

        // スクロール用
        int offset = 0;
        Point startPoint;

        // カーソル位置
        int cursorIndex = 0;

        //-------------------------------------------------------------------------------
        // コンストラクタ
        //-------------------------------------------------------------------------------
        public MainForm()
        {
            InitializeComponent();
            try
            {
                // 相場データの読み込み
                foreach (var line in File.ReadAllLines("../data.csv"))
                {
                    double value;
                    var t = line.TrimEnd(new[] {',', ' '}).Split(new[] {','});
                    if (t.Length >= 2 && double.TryParse(t[1], out value))
                    {
                        data.Add(new Tuple<string, double>(t[0], value));
                    }
                }

                // グラフ画像を生成
                chartPictureBox.Image = new Bitmap(chartPictureBox.Width, chartPictureBox.Height, PixelFormat.Format32bppArgb);
                DrawChart();
            }
            catch (IOException ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        //-------------------------------------------------------------------------------
        // 時間計測
        //-------------------------------------------------------------------------------

        // 計測開始・終了
        void startButton_Click(object sender, EventArgs e)  { stopWatch = Stopwatch.StartNew(); }
        void stopButton_Click(object sender, EventArgs e)   { stopWatch = null; }

        // 定期的に経過時間を更新
        void timer_Tick(object sender, EventArgs e)
        {
            if (stopWatch != null) timerLabel.Text = (int)stopWatch.Elapsed.TotalSeconds + " s";
        }

        //-------------------------------------------------------------------------------
        // マウス操作
        //-------------------------------------------------------------------------------

        // マウスがグラフ上で動いた
        void chartPictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button.HasFlag(MouseButtons.Left)) UpdateCursor(e.Location);
            if (e.Button.HasFlag(MouseButtons.Right))
            {
                offset += e.X - startPoint.X;
                startPoint = e.Location;
                DrawChart();
            }
        }

        // グラフ上でマウスボタンが押された
        void chartPictureBox_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button.HasFlag(MouseButtons.Left)) UpdateCursor(e.Location);
            if (e.Button.HasFlag(MouseButtons.Right)) startPoint = e.Location;
        }

        //-------------------------------------------------------------------------------
        // グラフ描画
        //-------------------------------------------------------------------------------

        // 座標計算
        Point GetPoint(int index) { return new Point(index + offset, 600 - (int)(data[index].Item2 * 3)); }

        // カーソル位置の更新
        void UpdateCursor(Point pt)
        {
            cursorIndex = pt.X - offset;
            if (cursorIndex < 0) cursorIndex = 0;
            if (data.Count <= cursorIndex) cursorIndex = data.Count - 1;
            chartLabel.Text = string.Format("{0} {1: 00.00}", data[cursorIndex].Item1, data[cursorIndex].Item2);
            DrawChart();
        }

        // 描画
        void DrawChart()
        {
            using (var g = Graphics.FromImage(chartPictureBox.Image))
            {
                // 画面をクリア
                g.Clear(Color.White);

                // 折れ線グラフ
                for (int i = 0; i < data.Count; i++)
                {
                    var pt = GetPoint(i);
                    g.DrawLine(pen, pt, new Point(pt.X, pt.Y + 1));
                }

                // カーソル位置をハイライト
                var p = GetPoint(cursorIndex);
                g.DrawLine(redPen, new Point(p.X, p.Y - 5), new Point(p.X, p.Y - 15));
            }
            // 再描画
            chartPictureBox.Refresh();
        }
    }
}